/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : AR_LIB_WIRE2.h
* Version      : Rev. 1
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements Arduino library.
* Creation Date: 2021/05/14
********************************************************************************/

#include "r_cg_userdefine.h"
#ifdef USE_WIRE2

#define     DUMMY_DATA  ( 0xFF )        /* dummy data to start receive          */

#define     IIC20_TRG   ( 0x0001 )      /* IIC20 and SO trigger data            */
#define     IIC20_CK    ( 0x0100 )      /* IIC20 CK trigger data                */
#define     TM11_TRG    ( 0x0002 )      /* TM11 count trigger data              */

#define     TX_DELAY2   ( 1 )           /* delay time for transmit end          */
#define     RX_DELAY2   ( 10 )          /* delay time for receive end           */


#define     BUS_FREE    ( 0x00 )        /* I2C bus is free                      */
#define     BUS_ERROR   ( 0x8F )        /* fail to get I2C bus                  */
#define     GET_BUS     ( 0x10 )        /* get I2C bus (start condition only)   */
#define     GET_BUS4TX  ( 0x20 )        /* get I2C bus for transmit             */
#define     TX_MODE     ( 0x30 )        /* data transmit mode                   */
#define     TX_END      ( 0x40 )        /* data transmit end                    */
#define     GET_BUS4RX  ( 0x50 )        /* get I2C bus for receive              */
#define     RX_MODE     ( 0x60 )        /* data receive mode                    */
#define     RX_END      ( 0x70 )        /* data receive end                     */
#define     BUFF_OVER   ( 0x81 )        /* data size over buffer size           */
#define     NO_SLAVE    ( 0x82 )        /* NACK for slave address               */
#define     NO_ACK      ( 0x83 )        /* no ACK for transmit data             */
#define     NO_DATA     ( 0x84 )        /* no data is requested                 */



/*==============================================================

    prototype of API functions

===============================================================*/

/*--------------------------------------------------------------
    IIC20(wire2)
---------------------------------------------------------------*/
// initialize IIC20 as master
void Wire2_begin(void);

// select slave to transmit data
uint8_t Wire2_beginTransmission(uint8_t saddr7);

// set a charactor to transmit buffer
uint8_t Wire2_writec(uint8_t data);

// set a block data to transmit buffer
uint8_t Wire2_writeb(uint8_t *buff, uint8_t bytes);

// start to transmit data stored in buffer
uint8_t Wire2_endTransmission(uint8_t stop);


// select slave to read block of data from I2C bus slave (select STOP condition or ReSTART)
uint8_t Wire2_requestFrom(uint8_t saddr7, uint16_t bytes, uint8_t stop);

// read block of data from I2C bus slave with STOP condition
uint8_t Wire2_requestFromS(uint8_t saddr7, uint16_t bytes);

// check available data number in receive buffer
uint8_t Wire2_available(void);

// get data from top of receive buffer
uint8_t Wire2_read(void);

#endif

