/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : AR_SKETCH.c
* Version      : V1.30
* Device(s)    : R5F104ML(G14-FPB)
* Tool-Chain   : CCRL
* Description  : This file implements the Arduino sketch.
* Creation Date: 2021/09/24
***********************************************************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include <AR_LIB_PORT.h>
#include <AR_LIB_TIME.h>
#include <AR_LIB_ANALOG.h>              // from V1.30
#include <AR_LIB_SHIFT.h>
#include <AR_SKETCH.h>

/* the following is for wire function   */
#ifdef USE_WIRE                         // from V1.30
#include <AR_LIB_WIRE.h>                // from V1.10
#include "AR_WIRE_DEF.h"
#endif

#ifdef USE_WIRE1
#include <AR_LIB_WIRE1.h>               // from V1.20
#endif
#ifdef USE_WIRE2
#include <AR_LIB_WIRE2.h>               // from V1.20
#endif

#ifdef USE_LCD                         // from V1.30
#include <LCD_LIB.h>
#endif

#include <stdarg.h>
#include <stdio.h>

/***********************************************************************
Pragma directive
************************************************************************/


/***********************************************************************
Global variables and functions
************************************************************************/

/*---------------------------------------------------------------------
     Definition area. Write pin definition here.
----------------------------------------------------------------------*/
#define SS_PIN    4                     // D4sHC595̃b`MiSPISSMj
#define SCK_PIN   7                     // D7sHC595̃VtgNbNM
#define  MOSI_PIN 8                     // D8sHC595̃VA͂


int VR_IN       = A0;                   // AiÓiŒŔj
int swPin       = 18;                   // D18̓I{[hSW_USER
int ex_swPin    = A1;                   // A1sOXCb`͂


/*---------------------------------------------------------------------
     Global variables
----------------------------------------------------------------------*/
const uint8_t LED_SEGMENT[] =
{   //ABCDEFGH  negative logic 0:ON/1:OFF
  0b00000011,   // segment data for 0
  0b10011111,   // segment data for 1
  0b00100101,   // segment data for 2
  0b00001101,   // segment data for 3
  0b10011001,   // segment data for 4
  0b01001001,   // segment data for 5
  0b01000001,   // segment data for 6
  0b00011111,   // segment data for 7
  0b00000001,   // segment data for 8
  0b00001001,   // segment data for 9
  0b11111111    // space
};

const uint8_t LED_DIGIT[] =
{
  0x8F,   // select 1st digit from left
  0x4F,   // select 2nd digit from left
  0x2F,   // select 3rd digit from left
  0x1F    // select 4th digit from left
};

int old_time = 0x0000;          // previous time(milli sec.)
int time_mode;

int analog_data;                          // ŒR킩̓d̓ǂݏol
int analog_work;                          // Ɨp
int ad_cnt  = 0;                          // ǂݏol̊i[|C^
int analog_buff[4];                       // ǂݏol̊i[obt@
int analog_sum;                           // 4񕪂̍vl
uint8_t blank_flag = 1;                   // 0TvbVEtO
uint8_t blank_mode = 1;                   // 0TuX [h

char prescale4sw = 0;                     // SW`FbN^C~OJE^( up to 5:20milli sec.)
int precount1 = 0x0000;                   // 100~bJEgp

int sw_data = 0xFFFF;                     // SW͗p


/***********************************************************************
* Sketch area
* Description  : This area is for sketch of arduino.
* setup        : setup program area executed once only
* loop         : main procedure
************************************************************************/

/*---------------------------------------------------------------------
     Arduino setup area. Write setup program here.
----------------------------------------------------------------------*/

void setup(void){
  // put your setup code here, to run once:
    pinMode(SS_PIN, OUTPUT);              // set D4pin to output mode
    pinMode(SCK_PIN, OUTPUT);             // set D7pin to output mode
    pinMode(MOSI_PIN, OUTPUT);            // set D8pin to output mode
    pinMode(ex_swPin, INPUT);             // set A1pin to input mode
    pinMode(swPin, INPUT);                // set D26pin to input mode

    digitalWrite( SCK_PIN, LOW );         // set SCK signal to LOW 

//  AiO͂̏ݒ

    analogReference(DEFAULT);
    analog_data = analogRead(VR_IN);      // AiOlǂ݂o
    analog_buff[0]  = analog_data;        // obt@̒lݒ
    analog_buff[1]  = analog_data;
    analog_buff[2]  = analog_data;
    analog_buff[3]  = analog_data;
    analog_sum  = ( analog_data << 2 );   // ώZl̏
    
}


/*---------------------------------------------------------------------
     Arduino loop area. Write the program in loop function here.
----------------------------------------------------------------------*/

void loop(void){
  // put your main code here, to run repeatedly:

    int time_work;                        // ݎԂ̃obt@
    uint8_t com_sel;                      // RMpobt@
    uint8_t seg_data;                     // ZOgMpobt@
    uint8_t sw_work;                      // SW̏Ԃ̍Ɨp
    int  dt_wk;                           // f[^̍Ɨp

/*--------------------------------------
    wait for 4milli seconds interval.
----------------------------------------*/

    time_work = ( int )( millis() & 0x0FFC ); // ~bԓǂݏo
    if ( old_time != time_work )          // 4~bo߂
    {

/*--------------------------------------
    4~bɕ\؂ւ
----------------------------------------*/

        old_time = time_work;           // ݎԂXV
        time_work &= 0x00C;             // 4~b̉2Zbg
    
        switch ( time_work )
        {
            case 0x00:                          // 1000̌
                dt_wk = ( analog_data / 1000 ); // 1000̌߂
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0\
                }
                else
                {
                  blank_flag = 0;               // 0TvX
                }
                com_sel = LED_DIGIT[0];         // IMZbg
                break;

            case 0x04:                          // 100IMZbg
                dt_wk = ( analog_data % 1000 ); // 1000̌폜
                dt_wk /= 100;                   // 100̌߂
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0\
                }
                else
                {
                  blank_flag = 0;               // 0TvX
                }
                com_sel = LED_DIGIT[1];         // IMZbg
                break;

            case 0x08:                          // 10IMZbg
                dt_wk = ( analog_data % 100 );  // 100ȏ̌폜
                dt_wk /= 10;                    // 10̌߂
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0\
                }
                else
                {
                  blank_flag = 0;               // 0TvX
                }
                com_sel = LED_DIGIT[2];         // IMZbg
                break;

            default:                            // 1̌
                dt_wk = ( analog_data % 10 );   // 1̌߂
                blank_flag = blank_mode;        // 0TvXݒ
                com_sel = LED_DIGIT[3];         // IMZbg

        }

        seg_data = LED_SEGMENT[dt_wk];          // ZOgEf[^ɕϊ

        digitalWrite( SS_PIN, LOW );            // SS signal(low level)I

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, seg_data); // ZOg𑗐M

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, com_sel);  // 𑗐M

        digitalWrite( SS_PIN, HIGH );           // SS signalIt(rise up)

/*--------------------------------------
    every 4milli seconds timing
    if count enable, then count time up.
----------------------------------------*/

        if ( 25 <= ++precount1 )
        {                               // 100~bo
            precount1 = 0;              // 4~bJE^NA

/*--------------------------------------
 * @ŐV̒l荞
 ----------------------------------------*/
            analog_data = analogRead(VR_IN);    // AiOlǂ݂o
            analog_sum -= analog_buff[ad_cnt];  // ƂÂf[^폜
            analog_sum += analog_data;          // ŐṼf[^Z
            analog_buff[ad_cnt] = analog_data;  // ŐṼf[^i[

            analog_data = ( analog_sum >> 2);   // ŐV̕ϒlݒ
            ++ad_cnt;                           // |C^XV
            ad_cnt &= 0x03;                     // ܂Ԃ

        }

/*--------------------------------------
    switch data check timing
----------------------------------------*/

        if ( ++prescale4sw >= 5 )       // check SW check timing or not
        {                               // 20milli seconds passing
            prescale4sw = 0x00;         // clear timing counter
            sw_data <<= 1;              // shift olddata left
            sw_data += ( digitalRead(ex_swPin)&digitalRead(swPin) ); // read sw data
            sw_work = (sw_data & 0x06); // extract check timing data
            if ( 0x04 == sw_work )
            {                           // sw is pushed
                time_mode ^= 0x01;      // change count mode flag
            }
        }
    
    }

}   /* end of loop function */


/*---------------------------------------------------------------------
     End of sketch area
----------------------------------------------------------------------*/

