/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : AR_SKETCH.c
* Version      : V1.30
* Device(s)    : R5F104ML(G14-FPB)
* Tool-Chain   : CCRL
* Description  : This file implements the Arduino sketch.
* Creation Date: 2021/09/24
***********************************************************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include <AR_LIB_PORT.h>
#include <AR_LIB_TIME.h>
#include <AR_LIB_ANALOG.h>              // from V1.30
#include <AR_LIB_SHIFT.h>
#include <AR_SKETCH.h>

/* the following is for wire function   */
#ifdef USE_WIRE                         // from V1.30
#include <AR_LIB_WIRE.h>                // from V1.10
#include "AR_WIRE_DEF.h"
#endif

#ifdef USE_WIRE1
#include <AR_LIB_WIRE1.h>               // from V1.20
#endif
#ifdef USE_WIRE2
#include <AR_LIB_WIRE2.h>               // from V1.20
#endif

#ifdef USE_LCD                         // from V1.30
#include <LCD_LIB.h>
#endif

#include <stdarg.h>
#include <stdio.h>

/***********************************************************************
Pragma directive
************************************************************************/


/***********************************************************************
Global variables and functions
************************************************************************/

/*---------------------------------------------------------------------
     Definition area. Write pin definition here.
----------------------------------------------------------------------*/
#define SS_PIN    4           // assign D4 pin to slave select pin for 7SEG_LED.
#define SCK_PIN   7           // assign D7 pin to serial clock pin for 7SEG_LED.
#define  MOSI_PIN 8           // assign D8 pin to serial data out pin for 7SEG_LED.



int swPin       = 18;                   // assign D18 pin to swPin for SW_USER.
int ex_swPin    = A1;                   // assign A1 pin to ex_SW


/*---------------------------------------------------------------------
     Global variables
----------------------------------------------------------------------*/
const uint8_t LED_SEGMENT[] =
{   //ABCDEFGH  negative logic 0:ON/1:OFF
  0b00000011,   // segment data for 0
  0b10011111,   // segment data for 1
  0b00100101,   // segment data for 2
  0b00001101,   // segment data for 3
  0b10011001,   // segment data for 4
  0b01001001,   // segment data for 5
  0b01000001,   // segment data for 6
  0b00011111,   // segment data for 7
  0b00000001,   // segment data for 8
  0b00001001,   // segment data for 9
  0b11111111    // space
};

const uint8_t LED_DIGIT[] =
{
  0x8F,   // select 1st digit from left
  0x4F,   // select 2nd digit from left
  0x2F,   // select 3rd digit from left
  0x1F    // select 4th digit from left
};

int old_time = 0x0000;          // previous time(milli sec.)

char mini_data = 0;             // minute data 0:1:3:7:F:1F:3F:7F:FF
char sec_data = 0;              // second count data(0 to 59)
char precount1 = 0;             // prescaler for 100milli seconds.(25)
char precount2 = 0;             // prescaler(10) for 1second
char digi_sel = 0;              // digit select

char prescale4sw = 0;           // prescaler for SW check( up to 5:20milli sec.)
int sw_data = 0xFFFF;           // SW data image(history of 20 milli sec. interval)
int time_mode = 0;              // o:stop/1:run


/***********************************************************************
* Sketch area
* Description  : This area is for sketch of arduino.
* setup        : setup program area executed once only
* loop         : main procedure
************************************************************************/

/*---------------------------------------------------------------------
     Arduino setup area. Write setup program here.
----------------------------------------------------------------------*/

void setup(void){
  // put your setup code here, to run once:
    pinMode(SS_PIN, OUTPUT);      // set D4pin to output mode
    pinMode(SCK_PIN, OUTPUT);     // set D7pin to output mode
    pinMode(MOSI_PIN, OUTPUT);    // set D8pin to output mode
    pinMode(swPin, INPUT);        // set D26pin to input mode

    digitalWrite( SCK_PIN, LOW ); // set SCK signal to LOW 
}


/*---------------------------------------------------------------------
     Arduino loop area. Write the program in loop function here.
----------------------------------------------------------------------*/

void loop(void){
  // put your main code here, to run repeatedly:

    int time_work;                      // present time buffer
    char com_sel;                       // common select signal
    char seg_data;                      // segment data
    char sw_work;                       // work for switch check
    uint8_t dt_wk;                         // time data work

/*--------------------------------------
    wait for 4milli seconds interval.
----------------------------------------*/

    time_work = ( int )( millis() & 0x0FFC ); // read milli sec data

    if ( old_time != time_work )        // check timing of change LED
    {

/*--------------------------------------
    every 4milli seconds timing
    change display digit.
----------------------------------------*/

        old_time = time_work;           // change current time data
        time_work &= 0x00C;             // get comsel data(= 4milli second unit)
    
        switch ( time_work )
        {
            case 0x00:                  // 10second digit timing
                dt_wk = ( sec_data / 10 );      // get 10second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of 10second digit data
                com_sel = LED_DIGIT[1];         // set 10second digit
                break;

            case 0x04:                  // second digit timing
                dt_wk = ( sec_data % 10 );      // get second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of second digit data
                seg_data &= 0xFE;               // set point
                com_sel = LED_DIGIT[2];         // set second digit
                break;

            case 0x08:                  // 100milli second digit timing
                dt_wk = precount2;              // get 100milli second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of 100milli second digit data
                com_sel = LED_DIGIT[3];         // set 100milli second digit
                break;

            default:                    // minute digit timing
                dt_wk = mini_data;              // get minute display data
                seg_data = LED_SEGMENT[dt_wk];  // get segment of minute display data
                com_sel = LED_DIGIT[0];         // set minute digit


        }

        digitalWrite( SS_PIN, LOW );      // turn on SS signal(low level)

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, seg_data); // segment data out

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, com_sel);  // digit data out


        digitalWrite( SS_PIN, HIGH ); // turn off SS signal(rise up)

/*--------------------------------------
    every 4milli seconds timing
    if count enable, then count time up.
----------------------------------------*/

        if (( 25 <= ++precount1 ) && ( 1 == time_mode ))
        {                               // 100 milli seconds passing
            precount1 = 0;              // clear 100milli second counter
            if (  10 <= ++precount2 )   // count for seconds
            {                           // one second passing
                precount2 = 0;          // clear one second counter
                if ( 60 <= ++sec_data )
                {                       // one minute passing
                    sec_data = 0;
                    if ( 9 == mini_data )
                    {                   // over flow
                        mini_data = 0;  // clear minute data
                    }
                    else
                    {                   // change minute data
                        ++mini_data;   // count minute data
                    }
                }
            }
        }

/*--------------------------------------
    switch data check timing
----------------------------------------*/

        if ( ++prescale4sw >= 5 )       // check SW check timing or not
        {                               // 20milli seconds passing
            prescale4sw = 0x00;         // clear timing counter
            sw_data <<= 1;              // shift olddata left
            sw_data += ( digitalRead(ex_swPin)&digitalRead(swPin) ); // read sw data
            sw_work = (sw_data & 0x06); // extract check timing data
            if ( 0x04 == sw_work )
            {                           // sw is pushed
                time_mode ^= 0x01;      // change count mode flag
            }
        }
    
    }

}   /* end of loop function */


/*---------------------------------------------------------------------
     End of sketch area
----------------------------------------------------------------------*/

